@@ -80,6 +80,7 @@ module Hledger.Data.Amount (
   -- ** arithmetic
   costOfMixedAmount,
   divideMixedAmount,
+  averageMixedAmounts,
   isNegativeMixedAmount,
   isZeroMixedAmount,
   isReallyZeroMixedAmount,
@@ -99,8 +100,10 @@ module Hledger.Data.Amount (
 ) where
 
 import Data.Char (isDigit)
-#ifndef DOUBLE
-import Data.Decimal
+#ifdef DOUBLE
+roundTo = flip const
+#else
+import Data.Decimal (roundTo)
 #endif
 import Data.Function (on)
 import Data.List
@@ -149,10 +152,8 @@ nullamt = amount
 missingamt :: Amount
 missingamt = amount{acommodity="AUTO"}
 
--- handy amount constructors for tests
-#ifdef DOUBLE
-roundTo = flip const
-#endif
+-- Handy amount constructors for tests.
+-- usd/eur/gbp round their argument to a whole number of pennies/cents.
 num n = amount{acommodity="",  aquantity=n}
 hrs n = amount{acommodity="h", aquantity=n,           astyle=amountstyle{asprecision=2, ascommodityside=R}}
 usd n = amount{acommodity="$", aquantity=roundTo 2 n, astyle=amountstyle{asprecision=2}}
@@ -183,7 +184,7 @@ amountWithCommodity c a = a{acommodity=c, aprice=NoPrice}
 
 -- | Convert an amount to the commodity of its assigned price, if any.  Notes:
 --
--- - price amounts must be MixedAmounts with exactly one component Amount (or there will be a runtime error)
+-- - price amounts must be MixedAmounts with exactly one component Amount (or there will be a runtime error) XXX
 --
 -- - price amounts should be positive, though this is not currently enforced
 costOfAmount :: Amount -> Amount
@@ -480,6 +481,11 @@ costOfMixedAmount (Mixed as) = Mixed $ map costOfAmount as
 divideMixedAmount :: MixedAmount -> Quantity -> MixedAmount
 divideMixedAmount (Mixed as) d = Mixed $ map (flip divideAmount d) as
 
+-- | Calculate the average of some mixed amounts.
+averageMixedAmounts :: [MixedAmount] -> MixedAmount
+averageMixedAmounts [] = 0
+averageMixedAmounts as = sum as `divideMixedAmount` fromIntegral (length as)
+
 -- | Is this mixed amount negative, if it can be normalised to a single commodity ?
 isNegativeMixedAmount :: MixedAmount -> Maybe Bool
 isNegativeMixedAmount m = case as of [a] -> Just $ isNegativeAmount a
