@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE NoMonoLocalBinds #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-|
@@ -63,18 +64,23 @@ module Hledger.Data.Dates (
 )
 where
 
-import Control.Applicative ((<*))
+import Prelude ()
+import Prelude.Compat
 import Control.Monad
-import Data.List
+import Data.List.Compat
 import Data.Maybe
+#if MIN_VERSION_time(1,5,0)
+import Data.Time.Format hiding (months)
+#else
 import Data.Time.Format
+import System.Locale (TimeLocale, defaultTimeLocale)
+#endif
 import Data.Time.Calendar
 import Data.Time.Calendar.OrdinalDate
 import Data.Time.Calendar.WeekDate
 import Data.Time.Clock
 import Data.Time.LocalTime
 import Safe (headMay, lastMay, readMay)
-import System.Locale (defaultTimeLocale)
 import Test.HUnit
 import Text.Parsec
 import Text.Printf
@@ -128,8 +134,8 @@ showDateSpan ds@(DateSpan (Just from) (Just to)) =
                          ,(2,[28,29])
                          ,(3,[31])
                          ,(4,[30])
-                         ,(5,[30])
-                         ,(6,[31])
+                         ,(5,[31])
+                         ,(6,[30])
                          ,(7,[31])
                          ,(8,[31])
                          ,(9,[30])
@@ -420,13 +426,23 @@ nthdayofweekcontaining n d | d1 >= d    = d1
 --     parseTime defaultTimeLocale "%Y-%m-%d %H:%M:%S" s
 --     ]
 
+parsetime :: ParseTime t => TimeLocale -> String -> String -> Maybe t
+parsetime =
+#if MIN_VERSION_time(1,5,0)
+     parseTimeM True
+#else
+     parseTime
+#endif
+
+
 -- | Parse a couple of date string formats to a time type.
 parsedateM :: String -> Maybe Day
 parsedateM s = firstJust [
-     parseTime defaultTimeLocale "%Y/%m/%d" s,
-     parseTime defaultTimeLocale "%Y-%m-%d" s
+     parsetime defaultTimeLocale "%Y/%m/%d" s,
+     parsetime defaultTimeLocale "%Y-%m-%d" s
      ]
 
+
 -- -- | Parse a date-time string to a time type, or raise an error.
 -- parsedatetime :: String -> LocalTime
 -- parsedatetime s = fromMaybe (error' $ "could not parse timestamp \"" ++ s ++ "\"")
@@ -440,7 +456,7 @@ parsedate s =  fromMaybe (error' $ "could not parse date \"" ++ s ++ "\"")
 -- | Parse a time string to a time type using the provided pattern, or
 -- return the default.
 parsetimewith :: ParseTime t => String -> String -> t -> t
-parsetimewith pat s def = fromMaybe def $ parseTime defaultTimeLocale pat s
+parsetimewith pat s def = fromMaybe def $ parsetime defaultTimeLocale pat s
 
 {-|
 Parse a date in any of the formats allowed in ledger's period expressions,
@@ -477,7 +493,7 @@ datesepchar :: Stream [Char] m Char => ParsecT [Char] st m Char
 datesepchar = oneOf datesepchars
 
 validYear, validMonth, validDay :: String -> Bool
-validYear s = length s >= 4 && isJust (readMay s :: Maybe Int)
+validYear s = length s >= 4 && isJust (readMay s :: Maybe Year)
 validMonth s = maybe False (\n -> n>=1 && n<=12) $ readMay s
 validDay s = maybe False (\n -> n>=1 && n<=31) $ readMay s
 
