@@ -106,7 +106,7 @@ instance Show Journal where
              (show accounts)
              (show $ jcommoditystyles j)
              -- ++ (show $ journalTransactions l)
-             where accounts = flatten $ journalAccountNameTree j
+             where accounts = filter (/= "root") $ flatten $ journalAccountNameTree j
 
 -- showJournalDebug j = unlines [
 --                       show j
@@ -212,9 +212,9 @@ journalAssetAccountQuery  :: Journal -> Query
 journalAssetAccountQuery _ = Acct "^assets?(:|$)"
 
 -- | A query for Liability accounts in this journal.
--- This is currently hard-coded to the case-insensitive regex @^liabilit(y|ies)(:|$)@.
+-- This is currently hard-coded to the case-insensitive regex @^(debts?|liabilit(y|ies))(:|$)@.
 journalLiabilityAccountQuery  :: Journal -> Query
-journalLiabilityAccountQuery _ = Acct "^liabilit(y|ies)(:|$)"
+journalLiabilityAccountQuery _ = Acct "^(debts?|liabilit(y|ies))(:|$)"
 
 -- | A query for Equity accounts in this journal.
 -- This is currently hard-coded to the case-insensitive regex @^equity(:|$)@.
@@ -392,10 +392,10 @@ journalApplyAliases aliases j@Journal{jtxns=ts} =
   --  else (dbgtrace $
   --        "applying additional command-line aliases:\n"
   --        ++ chomp (unlines $ map (" "++) $ lines $ ppShow aliases))) $
-  j{jtxns=map fixtransaction ts}
+  j{jtxns=map dotransaction ts}
     where
-      fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
-      fixposting p@Posting{paccount=a} = p{paccount=accountNameApplyAliases aliases a}
+      dotransaction t@Transaction{tpostings=ps} = t{tpostings=map doposting ps}
+      doposting p@Posting{paccount=a} = p{paccount= accountNameApplyAliases aliases a}
 
 -- | Do post-parse processing on a journal to make it ready for use: check
 -- all transactions balance, canonicalise amount formats, close any open
@@ -495,7 +495,7 @@ journalCanonicaliseAmounts :: Journal -> Journal
 journalCanonicaliseAmounts j@Journal{jtxns=ts} = j''
     where
       j'' = j'{jtxns=map fixtransaction ts}
-      j' = j{jcommoditystyles = canonicalStyles $ dbgAt 8 "journalAmounts" $ journalAmounts j}
+      j' = j{jcommoditystyles = canonicalStyles $ dbg8 "journalAmounts" $ journalAmounts j}
       fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
       fixposting p@Posting{pamount=a} = p{pamount=fixmixedamount a}
       fixmixedamount (Mixed as) = Mixed $ map fixamount as
@@ -674,7 +674,7 @@ Right samplejournal = journalBalanceTransactions $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/01/01",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="",
              tdescription="income",
              tcomment="",
@@ -690,7 +690,7 @@ Right samplejournal = journalBalanceTransactions $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/01",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="",
              tdescription="gift",
              tcomment="",
@@ -706,7 +706,7 @@ Right samplejournal = journalBalanceTransactions $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/02",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="",
              tdescription="save",
              tcomment="",
@@ -722,7 +722,7 @@ Right samplejournal = journalBalanceTransactions $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/03",
              tdate2=Nothing,
-             tstatus=True,
+             tstatus=Cleared,
              tcode="",
              tdescription="eat & shop",
              tcomment="",
@@ -738,7 +738,7 @@ Right samplejournal = journalBalanceTransactions $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/12/31",
              tdate2=Nothing,
-             tstatus=False,
+             tstatus=Uncleared,
              tcode="",
              tdescription="pay off",
              tcomment="",
