@@ -13,7 +13,7 @@ module Hledger.Data.Posting (
   posting,
   post,
   -- * operations
-  postingCleared,
+  postingStatus,
   isReal,
   isVirtual,
   isBalancedVirtual,
@@ -37,7 +37,6 @@ module Hledger.Data.Posting (
   joinAccountNames,
   concatAccountNames,
   accountNameApplyAliases,
-  accountNameApplyOneAlias,
   -- * arithmetic
   sumPostings,
   -- * rendering
@@ -68,7 +67,7 @@ nullposting, posting :: Posting
 nullposting = Posting
                 {pdate=Nothing
                 ,pdate2=Nothing
-                ,pstatus=False
+                ,pstatus=Uncleared
                 ,paccount=""
                 ,pamount=nullmixedamt
                 ,pcomment=""
@@ -138,14 +137,15 @@ postingDate2 p = headDef nulldate $ catMaybes dates
                 ,maybe Nothing (Just . tdate) $ ptransaction p
                 ]
 
--- |Is this posting cleared? If this posting was individually marked
--- as cleared, returns True. Otherwise, return the parent
--- transaction's cleared status or, if there is no parent
--- transaction, return False.
-postingCleared :: Posting -> Bool
-postingCleared p = if pstatus p
-                    then True
-                    else maybe False tstatus $ ptransaction p
+-- | Get a posting's cleared status: cleared or pending if those are
+-- explicitly set, otherwise the cleared status of its parent
+-- transaction, or uncleared if there is no parent transaction. (Note
+-- Uncleared's ambiguity, it can mean "uncleared" or "don't know".
+postingStatus :: Posting -> ClearedStatus
+postingStatus Posting{pstatus=s, ptransaction=mt}
+  | s == Uncleared = case mt of Just t  -> tstatus t
+                                Nothing -> Uncleared
+  | otherwise = s
 
 -- | Tags for this posting including any inherited from its parent transaction.
 postingAllTags :: Posting -> [Tag]
@@ -219,22 +219,27 @@ concatAccountNames :: [AccountName] -> AccountName
 concatAccountNames as = accountNameWithPostingType t $ intercalate ":" $ map accountNameWithoutPostingType as
     where t = headDef RegularPosting $ filter (/= RegularPosting) $ map accountNamePostingType as
 
--- | Rewrite an account name using all applicable aliases from the given list, in sequence.
+-- | Rewrite an account name using all matching aliases from the given list, in sequence.
+-- Each alias sees the result of applying the previous aliases.
 accountNameApplyAliases :: [AccountAlias] -> AccountName -> AccountName
 accountNameApplyAliases aliases a = accountNameWithPostingType atype aname'
   where
     (aname,atype) = (accountNameWithoutPostingType a, accountNamePostingType a)
-    matchingaliases = filter (\(re,_) -> regexMatchesCI re aname) aliases
-    aname' = foldl (flip (uncurry regexReplaceCI)) aname matchingaliases
+    aname' = foldl
+             (\acct alias -> dbg6 "result" $ aliasReplace (dbg6 "alias" alias) (dbg6 "account" acct))
+             aname
+             aliases
+
+-- aliasMatches :: AccountAlias -> AccountName -> Bool
+-- aliasMatches (BasicAlias old _) a = old `isAccountNamePrefixOf` a
+-- aliasMatches (RegexAlias re  _) a = regexMatchesCI re a
+
+aliasReplace :: AccountAlias -> AccountName -> AccountName
+aliasReplace (BasicAlias old new) a
+  | old `isAccountNamePrefixOf` a || old == a = new ++ drop (length old) a
+  | otherwise = a
+aliasReplace (RegexAlias re repl) a = regexReplaceCI re repl a
 
--- | Rewrite an account name using the first applicable alias from the given list, if any.
-accountNameApplyOneAlias :: [AccountAlias] -> AccountName -> AccountName
-accountNameApplyOneAlias aliases a = accountNameWithPostingType atype aname'
-  where
-    (aname,atype) = (accountNameWithoutPostingType a, accountNamePostingType a)
-    firstmatchingalias = headDef Nothing $ map Just $ filter (\(re,_) -> regexMatchesCI re aname) aliases
-    applyAlias = uncurry regexReplaceCI
-    aname' = maybe id applyAlias firstmatchingalias $ aname
 
 tests_Hledger_Data_Posting = TestList [
 
