@@ -64,7 +64,7 @@ nulltransaction = Transaction {
                     tsourcepos=nullsourcepos,
                     tdate=nulldate,
                     tdate2=Nothing,
-                    tstatus=False,
+                    tstatus=Uncleared,
                     tcode="",
                     tdescription="",
                     tcomment="",
@@ -102,14 +102,14 @@ tests_showTransactionUnelided = [
     nulltransaction{
       tdate=parsedate "2012/05/14",
       tdate2=Just $ parsedate "2012/05/15",
-      tstatus=False,
+      tstatus=Uncleared,
       tcode="code",
       tdescription="desc",
       tcomment="tcomment1\ntcomment2\n",
       ttags=[("ttag1","val1")],
       tpostings=[
         nullposting{
-          pstatus=True,
+          pstatus=Cleared,
           paccount="a",
           pamount=Mixed [usd 1, hrs 2],
           pcomment="\npcomment2\n",
@@ -140,7 +140,9 @@ showTransaction' elide t =
       date = showdate (tdate t) ++ maybe "" showedate (tdate2 t)
       showdate = printf "%-10s" . showDate
       showedate = printf "=%s" . showdate
-      status = if tstatus t then " *" else ""
+      status | tstatus t == Cleared = " *"
+             | tstatus t == Pending = " !"
+             | otherwise            = ""
       code = if length (tcode t) > 0 then printf " (%s)" $ tcode t else ""
       desc = if null d then "" else " " ++ d where d = tdescription t
       (samelinecomment, newlinecomments) =
@@ -184,7 +186,7 @@ postingAsLines elideamount ps p =
     showacct p =
       indent $ showstatus p ++ printf (printf "%%-%ds" w) (showAccountName Nothing (ptype p) (paccount p))
         where
-          showstatus p = if pstatus p then "* " else ""
+          showstatus p = if pstatus p == Cleared then "* " else ""
           w = maximum $ map (length . paccount) ps
     showamt =
         padleft 12 . showMixedAmount
@@ -194,7 +196,7 @@ tests_postingAsLines = [
     let p `gives` ls = assertEqual "" ls (postingAsLines False [p] p)
     posting `gives` ["                 0"]
     posting{
-      pstatus=True,
+      pstatus=Cleared,
       paccount="a",
       pamount=Mixed [usd 1, hrs 2],
       pcomment="pcomment1\npcomment2\n  tag3: val3  \n",
@@ -259,110 +261,136 @@ isTransactionBalanced styles t =
       canonicalise = maybe id canonicaliseMixedAmount styles
 
 -- | Ensure this transaction is balanced, possibly inferring a missing
--- amount or conversion price, or return an error message.
---
--- Balancing is affected by commodity display precisions, so those may
--- be provided.
---
--- We can infer a missing real amount when there are multiple real
--- postings and exactly one of them is amountless (likewise for
--- balanced virtual postings). Inferred amounts are converted to cost
--- basis when possible.
---
--- We can infer a conversion price when all real amounts are specified
--- and the sum of real postings' amounts is exactly two
--- non-explicitly-priced amounts in different commodities (likewise
--- for balanced virtual postings).
+-- amount or conversion price(s), or return an error message.
+-- Balancing is affected by commodity display precisions, so those can
+-- (optionally) be provided.
 balanceTransaction :: Maybe (Map.Map Commodity AmountStyle) -> Transaction -> Either String Transaction
-balanceTransaction styles t@Transaction{tpostings=ps}
-    | length rwithoutamounts > 1 || length bvwithoutamounts > 1
-        = Left $ printerr "could not balance this transaction (can't have more than one missing amount; remember to put 2 or more spaces before amounts)"
-    | not $ isTransactionBalanced styles t''' = Left $ printerr $ nonzerobalanceerror t'''
-    | otherwise = Right t''''
-    where
-      -- maybe infer missing amounts
-      (rwithamounts, rwithoutamounts)   = partition hasAmount $ realPostings t
-      (bvwithamounts, bvwithoutamounts) = partition hasAmount $ balancedVirtualPostings t
-      ramounts  = map pamount rwithamounts
-      bvamounts = map pamount bvwithamounts
-      t' = t{tpostings=map inferamount ps}
-          where
-            inferamount p | not (hasAmount p) && isReal p            = p{pamount = costOfMixedAmount (- sum ramounts)}
-                          | not (hasAmount p) && isBalancedVirtual p = p{pamount = costOfMixedAmount (- sum bvamounts)}
-                          | otherwise                             = p
-
-      -- maybe infer conversion prices, for real postings
-      rmixedamountsinorder = map pamount $ realPostings t'
-      ramountsinorder = concatMap amounts rmixedamountsinorder
-      rcommoditiesinorder  = map acommodity ramountsinorder
-      rsumamounts  = amounts $ sum rmixedamountsinorder
-      -- assumption: the sum of mixed amounts is normalised (one simple amount per commodity)
-      t'' = if length rsumamounts == 2 && all ((==NoPrice).aprice) rsumamounts && t'==t
-             then t'{tpostings=map inferprice ps}
-             else t'
-          where
-            -- assumption: a posting's mixed amount contains one simple amount
-            inferprice p@Posting{pamount=Mixed [a@Amount{acommodity=c,aprice=NoPrice}], ptype=RegularPosting}
-                = p{pamount=Mixed [a{aprice=conversionprice c}]}
-                where
-                  conversionprice c | c == unpricedcommodity
-                                        -- assign a balancing price. Use @@ for more exact output when possible.
-                                        -- invariant: prices should always be positive. Enforced with "abs"
-                                        = if length ramountsinunpricedcommodity == 1
-                                           then TotalPrice $ abs targetcommodityamount `withPrecision` maxprecision
-                                           else UnitPrice $ abs (targetcommodityamount `divideAmount` (aquantity unpricedamount)) `withPrecision` maxprecision
-                                    | otherwise = NoPrice
-                      where
-                        unpricedcommodity     = head $ filter (`elem` (map acommodity rsumamounts)) rcommoditiesinorder
-                        unpricedamount        = head $ filter ((==unpricedcommodity).acommodity) rsumamounts
-                        targetcommodityamount = head $ filter ((/=unpricedcommodity).acommodity) rsumamounts
-                        ramountsinunpricedcommodity = filter ((==unpricedcommodity).acommodity) ramountsinorder
-            inferprice p = p
-
-      -- maybe infer prices for balanced virtual postings. Just duplicates the above for now.
-      bvmixedamountsinorder = map pamount $ balancedVirtualPostings t''
-      bvamountsinorder = concatMap amounts bvmixedamountsinorder
-      bvcommoditiesinorder  = map acommodity bvamountsinorder
-      bvsumamounts  = amounts $ sum bvmixedamountsinorder
-      t''' = if length bvsumamounts == 2 && all ((==NoPrice).aprice) bvsumamounts && t'==t -- XXX could check specifically for bv amount inferring
-             then t''{tpostings=map inferprice ps}
-             else t''
-          where
-            inferprice p@Posting{pamount=Mixed [a@Amount{acommodity=c,aprice=NoPrice}], ptype=BalancedVirtualPosting}
-                = p{pamount=Mixed [a{aprice=conversionprice c}]}
-                where
-                  conversionprice c | c == unpricedcommodity
-                                        = if length bvamountsinunpricedcommodity == 1
-                                           then TotalPrice $ abs targetcommodityamount `withPrecision` maxprecision
-                                           else UnitPrice $ abs (targetcommodityamount `divideAmount` (aquantity unpricedamount)) `withPrecision` maxprecision
-                                    | otherwise = NoPrice
-                      where
-                        unpricedcommodity     = head $ filter (`elem` (map acommodity bvsumamounts)) bvcommoditiesinorder
-                        unpricedamount        = head $ filter ((==unpricedcommodity).acommodity) bvsumamounts
-                        targetcommodityamount = head $ filter ((/=unpricedcommodity).acommodity) bvsumamounts
-                        bvamountsinunpricedcommodity = filter ((==unpricedcommodity).acommodity) bvamountsinorder
-            inferprice p = p
-
-      -- tie the knot so eg relatedPostings works right
-      t'''' = txnTieKnot t'''
-
+balanceTransaction styles t =
+  case inferBalancingAmount t of
+    Left err -> Left err
+    Right t' -> let t'' = inferBalancingPrices t'
+                in if isTransactionBalanced styles t''
+                   then Right $ txnTieKnot t''
+                   else Left $ printerr $ nonzerobalanceerror t''
+     where
       printerr s = intercalate "\n" [s, showTransactionUnelided t]
+      nonzerobalanceerror :: Transaction -> String
+      nonzerobalanceerror t = printf "could not balance this transaction (%s%s%s)" rmsg sep bvmsg
+          where
+            (rsum, _, bvsum) = transactionPostingBalances t
+            rmsg | isReallyZeroMixedAmountCost rsum = ""
+                 | otherwise = "real postings are off by " ++ showMixedAmount (costOfMixedAmount rsum)
+            bvmsg | isReallyZeroMixedAmountCost bvsum = ""
+                  | otherwise = "balanced virtual postings are off by " ++ showMixedAmount (costOfMixedAmount bvsum)
+            sep = if not (null rmsg) && not (null bvmsg) then "; " else "" :: String
 
-nonzerobalanceerror :: Transaction -> String
-nonzerobalanceerror t = printf "could not balance this transaction (%s%s%s)" rmsg sep bvmsg
-    where
-      (rsum, _, bvsum) = transactionPostingBalances t
-      rmsg | isReallyZeroMixedAmountCost rsum = ""
-           | otherwise = "real postings are off by " ++ showMixedAmount (costOfMixedAmount rsum)
-      bvmsg | isReallyZeroMixedAmountCost bvsum = ""
-            | otherwise = "balanced virtual postings are off by " ++ showMixedAmount (costOfMixedAmount bvsum)
-      sep = if not (null rmsg) && not (null bvmsg) then "; " else "" :: String
+-- | Infer up to one missing amount for this transactions's real postings, and
+-- likewise for its balanced virtual postings, if needed; or return an error
+-- message if we can't.
+--
+-- We can infer a missing amount when there are multiple postings and exactly
+-- one of them is amountless. If the amounts had price(s) the inferred amount
+-- have the same price(s), and will be converted to the price commodity.
+-- 
+inferBalancingAmount :: Transaction -> Either String Transaction
+inferBalancingAmount t@Transaction{tpostings=ps}
+  | length amountlessrealps > 1
+      = Left $ printerr "could not balance this transaction - can't have more than one real posting with no amount (remember to put 2 or more spaces before amounts)"
+  | length amountlessbvps > 1
+      = Left $ printerr "could not balance this transaction - can't have more than one balanced virtual posting with no amount (remember to put 2 or more spaces before amounts)"
+  | otherwise
+      = Right t{tpostings=map inferamount ps}
+  where
+    printerr s = intercalate "\n" [s, showTransactionUnelided t]
+    ((amountfulrealps, amountlessrealps), realsum) = (partition hasAmount (realPostings t), sum $ map pamount amountfulrealps)
+    ((amountfulbvps, amountlessbvps), bvsum)       = (partition hasAmount (balancedVirtualPostings t), sum $ map pamount amountfulbvps)
+    inferamount p@Posting{ptype=RegularPosting}         | not (hasAmount p) = p{pamount=costOfMixedAmount (-realsum)}
+    inferamount p@Posting{ptype=BalancedVirtualPosting} | not (hasAmount p) = p{pamount=costOfMixedAmount (-bvsum)}
+    inferamount p = p
+
+-- | Infer prices for this transaction's posting amounts, if needed to make
+-- the postings balance, and if possible. This is done once for the real
+-- postings and again (separately) for the balanced virtual postings. When
+-- it's not possible, the transaction is left unchanged.
+-- 
+-- The simplest example is a transaction with two postings, each in a
+-- different commodity, with no prices specified. In this case we'll add a
+-- price to the first posting such that it can be converted to the commodity
+-- of the second posting (with -B), and such that the postings balance.
+-- 
+-- In general, we can infer a conversion price when the sum of posting amounts
+-- contains exactly two different commodities and no explicit prices.  Also
+-- all postings are expected to contain an explicit amount (no missing
+-- amounts) in a single commodity. Otherwise no price inferring is attempted.
+-- 
+-- The transaction itself could contain more than two commodities, and/or
+-- prices, if they cancel out; what matters is that the sum of posting amounts
+-- contains exactly two commodities and zero prices.
+-- 
+-- There can also be more than two postings in either of the commodities.
+-- 
+-- We want to avoid excessive display of digits when the calculated price is
+-- an irrational number, while hopefully also ensuring the displayed numbers
+-- make sense if the user does a manual calculation. This is (mostly) achieved
+-- in two ways:
+-- 
+-- - when there is only one posting in the "from" commodity, a total price
+--   (@@) is used, and all available decimal digits are shown
+-- 
+-- - otherwise, a suitable averaged unit price (@) is applied to the relevant
+--   postings, with display precision equal to the summed display precisions
+--   of the two commodities being converted between, or 2, whichever is larger.
+-- 
+-- (We don't always calculate a good-looking display precision for unit prices
+-- when the commodity display precisions are low, eg when a journal doesn't
+-- use any decimal places. The minimum of 2 helps make the prices shown by the
+-- print command a bit less surprising in this case. Could do better.)
+-- 
+inferBalancingPrices :: Transaction -> Transaction
+inferBalancingPrices t@Transaction{tpostings=ps} = t{tpostings=ps'}
+  where
+    ps' = map (priceInferrerFor t BalancedVirtualPosting) $
+          map (priceInferrerFor t RegularPosting) $
+          ps
+
+-- | Generate a posting update function which assigns a suitable balancing
+-- price to the posting, if and as appropriate for the given transaction and
+-- posting type (real or balanced virtual).
+priceInferrerFor :: Transaction -> PostingType -> (Posting -> Posting)
+priceInferrerFor t pt = inferprice
+  where
+    postings       = filter ((==pt).ptype) $ tpostings t
+    pmixedamounts  = map pamount postings
+    pamounts       = concatMap amounts pmixedamounts
+    pcommodities   = map acommodity pamounts
+    sumamounts     = amounts $ sum pmixedamounts -- sum normalises to one amount per commodity & price
+    sumcommodities = map acommodity sumamounts
+    sumprices      = filter (/=NoPrice) $ map aprice sumamounts
+    caninferprices = length sumcommodities == 2 && null sumprices
+
+    inferprice p@Posting{pamount=Mixed [a]}
+      | caninferprices && ptype p == pt && acommodity a == fromcommodity
+        = p{pamount=Mixed [a{aprice=conversionprice}]}
+      where
+        fromcommodity = head $ filter (`elem` sumcommodities) pcommodities -- these heads are ugly but should be safe
+        conversionprice
+          | fromcount==1 = TotalPrice $ abs toamount `withPrecision` maxprecision
+          | otherwise    = UnitPrice $ abs unitprice `withPrecision` unitprecision
+          where
+            fromcount     = length $ filter ((==fromcommodity).acommodity) pamounts
+            fromamount    = head $ filter ((==fromcommodity).acommodity) sumamounts
+            tocommodity   = head $ filter (/=fromcommodity) sumcommodities
+            toamount      = head $ filter ((==tocommodity).acommodity) sumamounts
+            unitprice     = toamount `divideAmount` (aquantity fromamount)
+            unitprecision = max 2 ((asprecision $ astyle $ toamount) + (asprecision $ astyle $ fromamount))
+    inferprice p = p
 
 -- Get a transaction's secondary date, defaulting to the primary date.
 transactionDate2 :: Transaction -> Day
 transactionDate2 t = fromMaybe (tdate t) $ tdate2 t
 
--- | Ensure a transaction's postings refer back to it.
+-- | Ensure a transaction's postings refer back to it, so that eg
+-- relatedPostings works right.
 txnTieKnot :: Transaction -> Transaction
 txnTieKnot t@Transaction{tpostings=ps} = t{tpostings=map (settxn t) ps}
 
@@ -382,7 +410,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,"    assets:checking"
         ,""
         ])
-       (let t = Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
+       (let t = Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
                 [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
                 ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
@@ -396,7 +424,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,"    assets:checking               $-47.18"
         ,""
         ])
-       (let t = Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
+       (let t = Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
                 [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
                 ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
@@ -412,7 +440,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
+        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
          ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.19)]}
          ] ""))
@@ -425,7 +453,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
+        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
          ] ""))
 
@@ -437,7 +465,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "coopportunity" "" []
+        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=missingmixedamt}
          ] ""))
 
@@ -450,7 +478,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2010/01/01") Nothing False "" "x" "" []
+        (txnTieKnot $ Transaction nullsourcepos (parsedate "2010/01/01") Nothing Uncleared "" "x" "" []
          [posting{paccount="a", pamount=Mixed [num 1 `at` (usd 2 `withPrecision` 0)]}
          ,posting{paccount="b", pamount= missingmixedamt}
          ] ""))
@@ -458,19 +486,19 @@ tests_Hledger_Data_Transaction = TestList $ concat [
   ,"balanceTransaction" ~: do
      assertBool "detect unbalanced entry, sign error"
                     (isLeft $ balanceTransaction Nothing
-                           (Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "test" "" []
+                           (Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "test" "" []
                             [posting{paccount="a", pamount=Mixed [usd 1]}
                             ,posting{paccount="b", pamount=Mixed [usd 1]}
                             ] ""))
 
      assertBool "detect unbalanced entry, multiple missing amounts"
                     (isLeft $ balanceTransaction Nothing
-                           (Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "test" "" []
+                           (Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "test" "" []
                             [posting{paccount="a", pamount=missingmixedamt}
                             ,posting{paccount="b", pamount=missingmixedamt}
                             ] ""))
 
-     let e = balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2007/01/28") Nothing False "" "" "" []
+     let e = balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1]}
                            ,posting{paccount="b", pamount=missingmixedamt}
                            ] "")
@@ -481,7 +509,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
                         Right e' -> (pamount $ last $ tpostings e')
                         Left _ -> error' "should not happen")
 
-     let e = balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing False "" "" "" []
+     let e = balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1.35]}
                            ,posting{paccount="b", pamount=Mixed [eur (-1)]}
                            ] "")
@@ -493,49 +521,49 @@ tests_Hledger_Data_Transaction = TestList $ concat [
                         Left _ -> error' "should not happen")
 
      assertBool "balanceTransaction balances based on cost if there are unit prices" (isRight $
-       balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing False "" "" "" []
+       balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1 `at` eur 2]}
                            ,posting{paccount="a", pamount=Mixed [usd (-2) `at` eur 1]}
                            ] ""))
 
      assertBool "balanceTransaction balances based on cost if there are total prices" (isRight $
-       balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing False "" "" "" []
+       balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1    @@ eur 1]}
                            ,posting{paccount="a", pamount=Mixed [usd (-2) @@ eur 1]}
                            ] ""))
 
   ,"isTransactionBalanced" ~: do
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ] ""
      assertBool "detect balanced" (isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.01)], ptransaction=Just t}
              ] ""
      assertBool "detect unbalanced" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ] ""
      assertBool "detect unbalanced, one posting" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 0], ptransaction=Just t}
              ] ""
      assertBool "one zero posting is considered balanced for now" (isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ,posting{paccount="d", pamount=Mixed [usd 100], ptype=VirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "virtual postings don't need to balance" (isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "balanced virtual postings need to balance among themselves" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing False "" "a" "" []
+     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
