@@ -15,6 +15,7 @@ module Hledger.Read (
        readJournal,
        readJournal',
        readJournalFile,
+       readJournalFiles,
        requireJournalFileExists,
        ensureJournalFileExists,
        -- * Parsers used elsewhere
@@ -25,20 +26,21 @@ module Hledger.Read (
        mamountp',
        numberp,
        codep,
+       accountaliasp,
        -- * Tests
        samplejournal,
        tests_Hledger_Read,
 )
 where
 import qualified Control.Exception as C
-import Control.Monad.Error
+import Control.Monad.Except
 import Data.List
 import Data.Maybe
 import System.Directory (doesFileExist, getHomeDirectory)
 import System.Environment (getEnv)
 import System.Exit (exitFailure)
 import System.FilePath ((</>))
-import System.IO (IOMode(..), withFile, stdin, stderr, hSetNewlineMode, universalNewlineMode)
+import System.IO (IOMode(..), openFile, stdin, stderr, hSetNewlineMode, universalNewlineMode)
 import Test.HUnit
 import Text.Printf
 
@@ -50,7 +52,7 @@ import Hledger.Read.TimelogReader as TimelogReader
 import Hledger.Read.CsvReader as CsvReader
 import Hledger.Utils
 import Prelude hiding (getContents, writeFile)
-import Hledger.Utils.UTF8IOCompat (getContents, hGetContents, writeFile)
+import Hledger.Utils.UTF8IOCompat (hGetContents, writeFile)
 
 
 journalEnvVar           = "LEDGER_FILE"
@@ -126,9 +128,9 @@ readJournal format rulesfile assrt path s =
         firstSuccessOrBestError :: [String] -> [Reader] -> IO (Either String Journal)
         firstSuccessOrBestError [] []        = return $ Left "no readers found"
         firstSuccessOrBestError errs (r:rs) = do
-          dbgAtM 1 "trying reader" (rFormat r)
-          result <- (runErrorT . (rParser r) rulesfile assrt path') s
-          dbgAtM 1 "reader result" $ either id show result
+          dbg1IO "trying reader" (rFormat r)
+          result <- (runExceptT . (rParser r) rulesfile assrt path') s
+          dbg1IO "reader result" $ either id show result
           case result of Right j -> return $ Right j                       -- success!
                          Left e  -> firstSuccessOrBestError (errs++[e]) rs -- keep trying
         firstSuccessOrBestError (e:_) []    = return $ Left e              -- none left, return first error
@@ -137,7 +139,7 @@ readJournal format rulesfile assrt path s =
 -- | Which readers are worth trying for this (possibly unspecified) format, filepath, and data ?
 readersFor :: (Maybe StorageFormat, Maybe FilePath, String) -> [Reader]
 readersFor (format,path,s) =
-    dbg ("possible readers for "++show (format,path,elideRight 30 s)) $
+    dbg1 ("possible readers for "++show (format,path,elideRight 30 s)) $
     case format of
      Just f  -> case readerForStorageFormat f of Just r  -> [r]
                                                  Nothing -> []
@@ -162,17 +164,24 @@ readersForPathAndData (f,s) = filter (\r -> (rDetector r) f s) readers
 -- conversion of that format. Also there is a flag specifying whether
 -- to check or ignore balance assertions in the journal.
 readJournalFile :: Maybe StorageFormat -> Maybe FilePath -> Bool -> FilePath -> IO (Either String Journal)
-readJournalFile format rulesfile assrt "-" = do
-  hSetNewlineMode stdin universalNewlineMode
-  getContents >>= readJournal format rulesfile assrt (Just "-")
-readJournalFile format rulesfile assrt f = do
-  requireJournalFileExists f
-  withFile f ReadMode $ \h -> do
+readJournalFile format rulesfile assrt f = readJournalFiles format rulesfile assrt [f]
+
+readJournalFiles :: Maybe StorageFormat -> Maybe FilePath -> Bool -> [FilePath] -> IO (Either String Journal)
+readJournalFiles format rulesfile assrt f = do
+  contents <- fmap concat $ mapM readFileAnyNewline f
+  readJournal format rulesfile assrt (listToMaybe f) contents
+ where
+  readFileAnyNewline f = do
+    requireJournalFileExists f
+    h <- fileHandle f
     hSetNewlineMode h universalNewlineMode
-    hGetContents h >>= readJournal format rulesfile assrt (Just f)
+    hGetContents h
+  fileHandle "-" = return stdin
+  fileHandle f = openFile f ReadMode
 
 -- | If the specified journal file does not exist, give a helpful error and quit.
 requireJournalFileExists :: FilePath -> IO ()
+requireJournalFileExists "-" = return ()
 requireJournalFileExists f = do
   exists <- doesFileExist f
   when (not exists) $ do
@@ -186,7 +195,7 @@ ensureJournalFileExists :: FilePath -> IO ()
 ensureJournalFileExists f = do
   exists <- doesFileExist f
   when (not exists) $ do
-    hPrintf stderr "Creating hledger journal file \"%s\".\n" f
+    hPrintf stderr "Creating hledger journal file %s.\n" f
     -- note Hledger.Utils.UTF8.* do no line ending conversion on windows,
     -- we currently require unix line endings on all platforms.
     newJournalContent >>= writeFile f
@@ -230,12 +239,12 @@ samplejournal = readJournal' $ unlines
 tests_Hledger_Read = TestList $
   tests_readJournal'
   ++ [
-   -- tests_Hledger_Read_JournalReader,
+   tests_Hledger_Read_JournalReader,
    tests_Hledger_Read_TimelogReader,
    tests_Hledger_Read_CsvReader,
 
    "journal" ~: do
-    r <- runErrorT $ parseWithCtx nullctx JournalReader.journal ""
+    r <- runExceptT $ parseWithCtx nullctx JournalReader.journal ""
     assertBool "journal should parse an empty file" (isRight $ r)
     jE <- readJournal Nothing Nothing True Nothing "" -- don't know how to get it from journal
     either error' (assertBool "journal parsing an empty file should give an empty journal" . null . jtxns) jE
