@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-|
 
 A reader for CSV data, using an extra rules file to help interpret the data.
@@ -19,22 +20,27 @@ module Hledger.Read.CsvReader (
   tests_Hledger_Read_CsvReader
 )
 where
-import Control.Applicative ((<$>), (<*))
+import Prelude ()
+import Prelude.Compat hiding (getContents)
 import Control.Exception hiding (try)
 import Control.Monad
-import Control.Monad.Error
+import Control.Monad.Except
 -- import Test.HUnit
 import Data.Char (toLower, isDigit, isSpace)
-import Data.List
+import Data.List.Compat
 import Data.Maybe
 import Data.Ord
 import Data.Time.Calendar (Day)
+#if MIN_VERSION_time(1,5,0)
+import Data.Time.Format (parseTimeM, defaultTimeLocale)
+#else
 import Data.Time.Format (parseTime)
+import System.Locale (defaultTimeLocale)
+#endif
 import Safe
 import System.Directory (doesFileExist)
 import System.FilePath
 import System.IO (stderr)
-import System.Locale (defaultTimeLocale)
 import Test.HUnit
 import Text.CSV (parseCSV, CSV)
 import Text.Parsec hiding (parse)
@@ -43,10 +49,9 @@ import Text.Parsec.Error
 import Text.Printf (hPrintf,printf)
 
 import Hledger.Data
-import Prelude hiding (getContents)
 import Hledger.Utils.UTF8IOCompat (getContents)
 import Hledger.Utils
-import Hledger.Read.JournalReader (amountp)
+import Hledger.Read.JournalReader (amountp, statusp)
 
 
 reader :: Reader
@@ -63,7 +68,7 @@ detect f s
 
 -- | Parse and post-process a "Journal" from CSV data, or give an error.
 -- XXX currently ignores the string and reads from the file path
-parse :: Maybe FilePath -> Bool -> FilePath -> String -> ErrorT String IO Journal
+parse :: Maybe FilePath -> Bool -> FilePath -> String -> ExceptT String IO Journal
 parse rulesfile _ f s = do
   r <- liftIO $ readJournalFromCsv rulesfile f s
   case r of Left e -> throwError e
@@ -92,11 +97,11 @@ readJournalFromCsv mrulesfile csvfile csvdata =
   if created
    then hPrintf stderr "creating default conversion rules file %s, edit this file for better results\n" rulesfile
    else hPrintf stderr "using conversion rules file %s\n" rulesfile
-  rules_ <- liftIO $ runErrorT $ parseRulesFile rulesfile
+  rules_ <- liftIO $ runExceptT $ parseRulesFile rulesfile
   let rules = case rules_ of
               Right (t::CsvRules) -> t
               Left err -> throwerr $ show err
-  dbgAtM 2 "rules" rules
+  dbg2IO "rules" rules
 
   -- apply skip directive
   let skip = maybe 0 oneorerror $ getDirective "skip" rules
@@ -108,10 +113,10 @@ readJournalFromCsv mrulesfile csvfile csvdata =
   -- parsec seems to fail if you pass it "-" here
   let parsecfilename = if csvfile == "-" then "(stdin)" else csvfile
   records <- (either throwerr id .
-              dbgAt 2 "validateCsv" . validateCsv skip .
-              dbgAt 2 "parseCsv")
+              dbg2 "validateCsv" . validateCsv skip .
+              dbg2 "parseCsv")
              `fmap` parseCsv parsecfilename csvdata
-  dbgAtM 1 "first 3 csv records" $ take 3 records
+  dbg1IO "first 3 csv records" $ take 3 records
 
   -- identify header lines
   -- let (headerlines, datalines) = identifyHeaderLines records
@@ -335,15 +340,15 @@ getDirective :: DirectiveName -> CsvRules -> Maybe FieldTemplate
 getDirective directivename = lookup directivename . rdirectives
 
 
-parseRulesFile :: FilePath -> ErrorT String IO CsvRules
+parseRulesFile :: FilePath -> ExceptT String IO CsvRules
 parseRulesFile f = do
   s <- liftIO $ (readFile' f >>= expandIncludes (takeDirectory f))
   let rules = parseCsvRules f s
   case rules of
-    Left e -> ErrorT $ return $ Left $ show e
+    Left e -> ExceptT $ return $ Left $ show e
     Right r -> do
-               r_ <- liftIO $ runErrorT $ validateRules r
-               ErrorT $ case r_ of
+               r_ <- liftIO $ runExceptT $ validateRules r
+               ExceptT $ case r_ of
                  Left e -> return $ Left $ show $ toParseError e
                  Right r -> return $ Right r
   where
@@ -369,13 +374,13 @@ parseCsvRules rulesfile s =
   runParser rulesp rules rulesfile s
 
 -- | Return the validated rules, or an error.
-validateRules :: CsvRules -> ErrorT String IO CsvRules
+validateRules :: CsvRules -> ExceptT String IO CsvRules
 validateRules rules = do
-  unless (isAssigned "date")   $ ErrorT $ return $ Left "Please specify (at top level) the date field. Eg: date %1\n"
+  unless (isAssigned "date")   $ ExceptT $ return $ Left "Please specify (at top level) the date field. Eg: date %1\n"
   unless ((amount && not (amountin || amountout)) ||
           (not amount && (amountin && amountout)))
-    $ ErrorT $ return $ Left "Please specify (at top level) either the amount field, or both the amount-in and amount-out fields. Eg: amount %2\n"
-  ErrorT $ return $ Right rules
+    $ ExceptT $ return $ Left "Please specify (at top level) either the amount field, or both the amount-in and amount-out fields. Eg: amount %2\n"
+  ExceptT $ return $ Right rules
   where
     amount = isAssigned "amount"
     amountin = isAssigned "amount-in"
@@ -410,7 +415,7 @@ commentline :: Stream [Char] m t => ParsecT [Char] CsvRules m ()
 commentline = many spacenonewline >> commentchar >> restofline >> return () <?> "comment line"
 
 commentchar :: Stream [Char] m t => ParsecT [Char] CsvRules m Char
-commentchar = oneOf ";#"
+commentchar = oneOf ";#*"
 
 directive :: Stream [Char] m t => ParsecT [Char] CsvRules m (DirectiveName, String)
 directive = do
@@ -597,7 +602,15 @@ transactionFromCsvRecord sourcepos rules record = t
        ++"or "++maybe "add a" (const "change your") mskip++" skip rule"
       ,"for m/d/y or d/m/y dates, use date-format %-m/%-d/%Y or date-format %-d/%-m/%Y"
       ]
-    status      = maybe False ((=="*") . render) $ mfieldtemplate "status"
+    status      =
+      case mfieldtemplate "status" of
+        Nothing  -> Uncleared
+        Just str -> either statuserror id $ runParser (statusp <* eof) nullctx "" $ render str
+          where
+            statuserror err = error' $ unlines
+              ["error: could not parse \""++str++"\" as a cleared status (should be *, ! or empty)"
+              ,"the parse error is:      "++show err
+              ]
     code        = maybe "" render $ mfieldtemplate "code"
     description = maybe "" render $ mfieldtemplate "description"
     comment     = maybe "" render $ mfieldtemplate "comment"
@@ -713,7 +726,13 @@ renderTemplate rules record t = regexReplaceBy "%[A-z0-9]+" replace t
 parseDateWithFormatOrDefaultFormats :: Maybe DateFormat -> String -> Maybe Day
 parseDateWithFormatOrDefaultFormats mformat s = firstJust $ map parsewith formats
   where
-    parsewith = flip (parseTime defaultTimeLocale) s
+    parsetime =
+#if MIN_VERSION_time(1,5,0)
+     parseTimeM True
+#else
+     parseTime
+#endif
+    parsewith = flip (parsetime defaultTimeLocale) s
     formats = maybe
                ["%Y/%-m/%-d"
                ,"%Y-%-m-%-d"
