@@ -30,32 +30,38 @@ module Hledger.Read.JournalReader (
   datetimep,
   codep,
   accountnamep,
+  modifiedaccountname,
   postingp,
   amountp,
   amountp',
   mamountp',
   numberp,
+  statusp,
   emptyorcommentlinep,
-  followingcommentp
-#ifdef TESTS
+  followingcommentp,
+  accountaliasp
   -- * Tests
+  ,tests_Hledger_Read_JournalReader
+#ifdef TESTS
   -- disabled by default, HTF not available on windows
   ,htf_thisModulesTests
   ,htf_Hledger_Read_JournalReader_importedTests
 #endif
 )
 where
-import Control.Applicative ((<*))
+import Prelude ()
+import Prelude.Compat hiding (readFile)
 import qualified Control.Exception as C
-import Control.Monad
-import Control.Monad.Error
+import Control.Monad.Compat
+import Control.Monad.Except (ExceptT(..), liftIO, runExceptT, throwError, catchError)
 import Data.Char (isNumber)
-import Data.List
+import Data.List.Compat
 import Data.List.Split (wordsBy)
 import Data.Maybe
 import Data.Time.Calendar
 import Data.Time.LocalTime
 import Safe (headDef, lastDef)
+import Test.HUnit
 #ifdef TESTS
 import Test.Framework
 import Text.Parsec.Error
@@ -67,7 +73,6 @@ import System.Time (getClockTime)
 
 import Hledger.Data
 import Hledger.Utils
-import Prelude hiding (readFile)
 
 
 -- standard reader exports
@@ -87,7 +92,7 @@ detect f s
 
 -- | Parse and post-process a "Journal" from hledger's journal file
 -- format, or give an error.
-parse :: Maybe FilePath -> Bool -> FilePath -> String -> ErrorT String IO Journal
+parse :: Maybe FilePath -> Bool -> FilePath -> String -> ExceptT String IO Journal
 parse _ = parseJournalWith journal
 
 -- parsing utils
@@ -98,7 +103,7 @@ combineJournalUpdates us = liftM (foldl' (\acc new x -> new (acc x)) id) $ seque
 
 -- | Given a JournalUpdate-generating parsec parser, file path and data string,
 -- parse and post-process a Journal so that it's ready to use, or give an error.
-parseJournalWith :: (ParsecT [Char] JournalContext (ErrorT String IO) (JournalUpdate,JournalContext)) -> Bool -> FilePath -> String -> ErrorT String IO Journal
+parseJournalWith :: (ParsecT [Char] JournalContext (ExceptT String IO) (JournalUpdate,JournalContext)) -> Bool -> FilePath -> String -> ExceptT String IO Journal
 parseJournalWith p assrt f s = do
   tc <- liftIO getClockTime
   tl <- liftIO getCurrentLocalTime
@@ -151,7 +156,7 @@ clearAccountAliases = modifyState (\(ctx@Ctx{..}) -> ctx{ctxAliases=[]})
 -- | Top-level journal parser. Returns a single composite, I/O performing,
 -- error-raising "JournalUpdate" (and final "JournalContext") which can be
 -- applied to an empty journal to get the final result.
-journal :: ParsecT [Char] JournalContext (ErrorT String IO) (JournalUpdate,JournalContext)
+journal :: ParsecT [Char] JournalContext (ExceptT String IO) (JournalUpdate,JournalContext)
 journal = do
   journalupdates <- many journalItem
   eof
@@ -171,7 +176,7 @@ journal = do
                            ] <?> "journal transaction or directive"
 
 -- cf http://ledger-cli.org/3.0/doc/ledger3.html#Command-Directives
-directive :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+directive :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 directive = do
   optional $ char '!'
   choice' [
@@ -189,7 +194,7 @@ directive = do
    ]
   <?> "directive"
 
-includedirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+includedirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 includedirective = do
   string "include"
   many1 spacenonewline
@@ -197,7 +202,7 @@ includedirective = do
   outerState <- getState
   outerPos <- getPosition
   let curdir = takeDirectory (sourceName outerPos)
-  let (u::ErrorT String IO (Journal -> Journal, JournalContext)) = do
+  let (u::ExceptT String IO (Journal -> Journal, JournalContext)) = do
        filepath <- expandPath curdir filename
        txt <- readFileOrError outerPos filepath
        let inIncluded = show outerPos ++ " in included file " ++ show filename ++ ":\n"
@@ -210,18 +215,18 @@ includedirective = do
                             return (u, ctx)
          Left err -> throwError $ inIncluded ++ show err
        where readFileOrError pos fp =
-                ErrorT $ liftM Right (readFile' fp) `C.catch`
+                ExceptT $ liftM Right (readFile' fp) `C.catch`
                   \e -> return $ Left $ printf "%s reading %s:\n%s" (show pos) fp (show (e::C.IOException))
-  r <- liftIO $ runErrorT u
+  r <- liftIO $ runExceptT u
   case r of
     Left err -> return $ throwError err
-    Right (ju, _finalparsectx) -> return $ ErrorT $ return $ Right ju
+    Right (ju, _finalparsectx) -> return $ ExceptT $ return $ Right ju
 
 journalAddFile :: (FilePath,String) -> Journal -> Journal
 journalAddFile f j@Journal{files=fs} = j{files=fs++[f]}
  -- NOTE: first encountered file to left, to avoid a reverse
 
-accountdirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+accountdirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 accountdirective = do
   string "account"
   many1 spacenonewline
@@ -229,33 +234,54 @@ accountdirective = do
   newline
   pushParentAccount parent
   -- return $ return id
-  return $ ErrorT $ return $ Right id
+  return $ ExceptT $ return $ Right id
 
-enddirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+enddirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 enddirective = do
   string "end"
   popParentAccount
   -- return (return id)
-  return $ ErrorT $ return $ Right id
+  return $ ExceptT $ return $ Right id
 
-aliasdirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+aliasdirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 aliasdirective = do
   string "alias"
   many1 spacenonewline
-  orig <- many1 $ noneOf "="
-  char '='
-  alias <- restofline
-  addAccountAlias (accountNameWithoutPostingType $ strip orig
-                  ,accountNameWithoutPostingType $ strip alias)
+  alias <- accountaliasp
+  addAccountAlias alias
   return $ return id
 
-endaliasesdirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+accountaliasp :: Stream [Char] m Char => ParsecT [Char] st m AccountAlias
+accountaliasp = regexaliasp <|> basicaliasp
+
+basicaliasp :: Stream [Char] m Char => ParsecT [Char] st m AccountAlias
+basicaliasp = do
+  -- pdbg 0 "basicaliasp"
+  old <- rstrip <$> (many1 $ noneOf "=")
+  char '='
+  many spacenonewline
+  new <- rstrip <$> anyChar `manyTill` eolof  -- don't require a final newline, good for cli options
+  return $ BasicAlias old new
+
+regexaliasp :: Stream [Char] m Char => ParsecT [Char] st m AccountAlias
+regexaliasp = do
+  -- pdbg 0 "regexaliasp"
+  char '/'
+  re <- many1 $ noneOf "/\n\r" -- paranoid: don't try to read past line end
+  char '/'
+  many spacenonewline
+  char '='
+  many spacenonewline
+  repl <- rstrip <$> anyChar `manyTill` eolof
+  return $ RegexAlias re repl
+
+endaliasesdirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 endaliasesdirective = do
   string "end aliases"
   clearAccountAliases
   return (return id)
 
-tagdirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+tagdirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 tagdirective = do
   string "tag" <?> "tag directive"
   many1 spacenonewline
@@ -263,13 +289,13 @@ tagdirective = do
   restofline
   return $ return id
 
-endtagdirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+endtagdirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 endtagdirective = do
   (string "end tag" <|> string "pop") <?> "end tag or pop directive"
   restofline
   return $ return id
 
-defaultyeardirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+defaultyeardirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 defaultyeardirective = do
   char 'Y' <?> "default year"
   many spacenonewline
@@ -279,7 +305,7 @@ defaultyeardirective = do
   setYear y'
   return $ return id
 
-defaultcommoditydirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+defaultcommoditydirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 defaultcommoditydirective = do
   char 'D' <?> "default commodity"
   many1 spacenonewline
@@ -288,7 +314,7 @@ defaultcommoditydirective = do
   restofline
   return $ return id
 
-historicalpricedirective :: ParsecT [Char] JournalContext (ErrorT String IO) HistoricalPrice
+historicalpricedirective :: ParsecT [Char] JournalContext (ExceptT String IO) HistoricalPrice
 historicalpricedirective = do
   char 'P' <?> "historical price"
   many spacenonewline
@@ -300,7 +326,7 @@ historicalpricedirective = do
   restofline
   return $ HistoricalPrice date symbol price
 
-ignoredpricecommoditydirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+ignoredpricecommoditydirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 ignoredpricecommoditydirective = do
   char 'N' <?> "ignored-price commodity"
   many1 spacenonewline
@@ -308,7 +334,7 @@ ignoredpricecommoditydirective = do
   restofline
   return $ return id
 
-commodityconversiondirective :: ParsecT [Char] JournalContext (ErrorT String IO) JournalUpdate
+commodityconversiondirective :: ParsecT [Char] JournalContext (ExceptT String IO) JournalUpdate
 commodityconversiondirective = do
   char 'C' <?> "commodity conversion"
   many1 spacenonewline
@@ -320,7 +346,7 @@ commodityconversiondirective = do
   restofline
   return $ return id
 
-modifiertransaction :: ParsecT [Char] JournalContext (ErrorT String IO) ModifierTransaction
+modifiertransaction :: ParsecT [Char] JournalContext (ExceptT String IO) ModifierTransaction
 modifiertransaction = do
   char '=' <?> "modifier transaction"
   many spacenonewline
@@ -328,7 +354,7 @@ modifiertransaction = do
   postings <- postings
   return $ ModifierTransaction valueexpr postings
 
-periodictransaction :: ParsecT [Char] JournalContext (ErrorT String IO) PeriodicTransaction
+periodictransaction :: ParsecT [Char] JournalContext (ExceptT String IO) PeriodicTransaction
 periodictransaction = do
   char '~' <?> "periodic transaction"
   many spacenonewline
@@ -337,14 +363,14 @@ periodictransaction = do
   return $ PeriodicTransaction periodexpr postings
 
 -- | Parse a (possibly unbalanced) transaction.
-transaction :: ParsecT [Char] JournalContext (ErrorT String IO) Transaction
+transaction :: ParsecT [Char] JournalContext (ExceptT String IO) Transaction
 transaction = do
   -- ptrace "transaction"
   sourcepos <- getPosition
   date <- datep <?> "transaction"
   edate <- optionMaybe (secondarydatep date) <?> "secondary date"
   lookAhead (spacenonewline <|> newline) <?> "whitespace or newline"
-  status <- status <?> "cleared flag"
+  status <- statusp <?> "cleared status"
   code <- codep <?> "transaction code"
   description <- descriptionp >>= return . strip
   comment <- try followingcommentp <|> (newline >> return "")
@@ -384,14 +410,14 @@ test_transaction = do
      nulltransaction{
       tdate=parsedate "2012/05/14",
       tdate2=Just $ parsedate "2012/05/15",
-      tstatus=False,
+      tstatus=Uncleared,
       tcode="code",
       tdescription="desc",
       tcomment=" tcomment1\n tcomment2\n ttag1: val1\n",
       ttags=[("ttag1","val1")],
       tpostings=[
         nullposting{
-          pstatus=True,
+          pstatus=Cleared,
           paccount="a",
           pamount=Mixed [usd 1],
           pcomment=" pcomment1\n pcomment2\n ptag1: val1\n  ptag2: val2\n",
@@ -402,6 +428,13 @@ test_transaction = do
         ],
       tpreceding_comment_lines=""
       }
+    unlines [
+      "2015/1/1",
+      ]
+     `gives`
+     nulltransaction{
+      tdate=parsedate "2015/01/01",
+      }
 
     assertRight $ parseWithCtx nullctx transaction $ unlines
       ["2007/01/28 coopportunity"
@@ -440,8 +473,10 @@ test_transaction = do
     assertEqual 2 (let Right t = p in length $ tpostings t)
 #endif
 
--- | Parse a date in YYYY/MM/DD format. Fewer digits are allowed. The year
--- may be omitted if a default year has already been set.
+-- | Parse a date in YYYY/MM/DD format.
+-- Hyphen (-) and period (.) are also allowed as separators.
+-- The year may be omitted if a default year has been set.
+-- Leading zeroes may be omitted.
 datep :: Stream [Char] m t => ParsecT [Char] JournalContext m Day
 datep = do
   -- hacky: try to ensure precise errors for invalid dates
@@ -463,10 +498,12 @@ datep = do
     Just date -> return date
   <?> "full or partial date"
 
--- | Parse a date and time in YYYY/MM/DD HH:MM[:SS][+-ZZZZ] format.  Any
--- timezone will be ignored; the time is treated as local time.  Fewer
--- digits are allowed, except in the timezone. The year may be omitted if
--- a default year has already been set.
+-- | Parse a date and time in YYYY/MM/DD HH:MM[:SS][+-ZZZZ] format.
+-- Hyphen (-) and period (.) are also allowed as date separators.
+-- The year may be omitted if a default year has been set.
+-- Seconds are optional.
+-- The timezone is optional and ignored (the time is always interpreted as a local time).
+-- Leading zeroes may be omitted (except in a timezone).
 datetimep :: Stream [Char] m Char => ParsecT [Char] JournalContext m LocalTime
 datetimep = do
   day <- datep
@@ -508,15 +545,21 @@ secondarydatep primarydate = do
   edate <- withDefaultYear primarydate datep
   return edate
 
-status :: Stream [Char] m Char => ParsecT [Char] JournalContext m Bool
-status = try (do { many spacenonewline; (char '*' <|> char '!') <?> "status"; return True } ) <|> return False
+statusp :: Stream [Char] m Char => ParsecT [Char] JournalContext m ClearedStatus
+statusp =
+  choice'
+    [ many spacenonewline >> char '*' >> return Cleared
+    , many spacenonewline >> char '!' >> return Pending
+    , return Uncleared
+    ]
+    <?> "cleared status"
 
 codep :: Stream [Char] m Char => ParsecT [Char] JournalContext m String
 codep = try (do { many1 spacenonewline; char '(' <?> "codep"; code <- anyChar `manyTill` char ')'; return code } ) <|> return ""
 
 -- Parse the following whitespace-beginning lines as postings, posting tags, and/or comments.
 postings :: Stream [Char] m Char => ParsecT [Char] JournalContext m [Posting]
-postings = many1 (try postingp) <?> "postings"
+postings = many (try postingp) <?> "postings"
 
 -- linebeginningwithspaces :: Stream [Char] m Char => ParsecT [Char] JournalContext m String
 -- linebeginningwithspaces = do
@@ -528,7 +571,7 @@ postings = many1 (try postingp) <?> "postings"
 postingp :: Stream [Char] m Char => ParsecT [Char] JournalContext m Posting
 postingp = do
   many1 spacenonewline
-  status <- status
+  status <- statusp
   many spacenonewline
   account <- modifiedaccountname
   let (ptype, account') = (accountNamePostingType account, unbracket account)
@@ -618,21 +661,24 @@ modifiedaccountname = do
   aliases <- getAccountAliases
   return $ accountNameApplyAliases aliases prefixed
 
--- | Parse an account name. Account names may have single spaces inside
--- them, and are terminated by two or more spaces. They should have one or
--- more components of at least one character, separated by the account
--- separator char.
+-- | Parse an account name. Account names start with a non-space, may
+-- have single spaces inside them, and are terminated by two or more
+-- spaces (or end of input). Also they have one or more components of
+-- at least one character, separated by the account separator char.
+-- (This parser will also consume one following space, if present.)
 accountnamep :: Stream [Char] m Char => ParsecT [Char] st m AccountName
 accountnamep = do
-    a <- many1 (nonspace <|> singlespace)
-    let a' = striptrailingspace a
-    when (accountNameFromComponents (accountNameComponents a') /= a')
-         (fail $ "account name seems ill-formed: "++a')
-    return a'
+    a <- do
+      c <- nonspace
+      cs <- striptrailingspace <$> many (nonspace <|> singlespace)
+      return $ c:cs
+    when (accountNameFromComponents (accountNameComponents a) /= a)
+         (fail $ "account name seems ill-formed: "++a)
+    return a
     where
       singlespace = try (do {spacenonewline; do {notFollowedBy spacenonewline; return ' '}})
-      -- couldn't avoid consuming a final space sometimes, harmless
-      striptrailingspace s = if last s == ' ' then init s else s
+      striptrailingspace "" = ""
+      striptrailingspace s  = if last s == ' ' then init s else s
 
 -- accountnamechar = notFollowedBy (oneOf "()[]") >> nonspace
 --     <?> "account name character (non-bracket, non-parenthesis, non-whitespace)"
@@ -815,7 +861,7 @@ numberp = do
   -- ptrace "numberp"
   sign <- signp
   parts <- many1 $ choice' [many1 digit, many1 $ char ',', many1 $ char '.']
-  dbgAt 8 "numberp parsed" (sign,parts) `seq` return ()
+  dbg8 "numberp parsed" (sign,parts) `seq` return ()
 
   -- check the number is well-formed and identify the decimal point and digit
   -- group separator characters used, if any
@@ -853,40 +899,38 @@ numberp = do
       frac' = if null frac then "0" else frac
       quantity = read $ sign++int'++"."++frac' -- this read should never fail
 
-  return $ dbgAt 8 "numberp quantity,precision,mdecimalpoint,mgrps" (quantity,precision,mdecimalpoint,mgrps)
+  return $ dbg8 "numberp quantity,precision,mdecimalpoint,mgrps" (quantity,precision,mdecimalpoint,mgrps)
   <?> "numberp"
   where
     numeric = isNumber . headDef '_'
 
-#ifdef TESTS
-test_numberp = do
-      let s `is` n = assertParseEqual' (parseWithCtx nullctx numberp s) n
-          assertFails = assertBool . isLeft . parseWithCtx nullctx numberp
-      assertFails ""
-      "0"          `is` (0, 0, '.', ',', [])
-      "1"          `is` (1, 0, '.', ',', [])
-      "1.1"        `is` (1.1, 1, '.', ',', [])
-      "1,000.1"    `is` (1000.1, 1, '.', ',', [3])
-      "1.00.000,1" `is` (100000.1, 1, ',', '.', [3,2])
-      "1,000,000"  `is` (1000000, 0, '.', ',', [3,3])
-      "1."         `is` (1,   0, '.', ',', [])
-      "1,"         `is` (1,   0, ',', '.', [])
-      ".1"         `is` (0.1, 1, '.', ',', [])
-      ",1"         `is` (0.1, 1, ',', '.', [])
-      assertFails "1,000.000,1"
-      assertFails "1.000,000.1"
-      assertFails "1,000.000.1"
-      assertFails "1,,1"
-      assertFails "1..1"
-      assertFails ".1,"
-      assertFails ",1."
-#endif
+-- test_numberp = do
+--       let s `is` n = assertParseEqual (parseWithCtx nullctx numberp s) n
+--           assertFails = assertBool . isLeft . parseWithCtx nullctx numberp
+--       assertFails ""
+--       "0"          `is` (0, 0, '.', ',', [])
+--       "1"          `is` (1, 0, '.', ',', [])
+--       "1.1"        `is` (1.1, 1, '.', ',', [])
+--       "1,000.1"    `is` (1000.1, 1, '.', ',', [3])
+--       "1.00.000,1" `is` (100000.1, 1, ',', '.', [3,2])
+--       "1,000,000"  `is` (1000000, 0, '.', ',', [3,3])
+--       "1."         `is` (1,   0, '.', ',', [])
+--       "1,"         `is` (1,   0, ',', '.', [])
+--       ".1"         `is` (0.1, 1, '.', ',', [])
+--       ",1"         `is` (0.1, 1, ',', '.', [])
+--       assertFails "1,000.000,1"
+--       assertFails "1.000,000.1"
+--       assertFails "1,000.000.1"
+--       assertFails "1,,1"
+--       assertFails "1..1"
+--       assertFails ".1,"
+--       assertFails ",1."
 
 -- comment parsers
 
 multilinecommentp :: Stream [Char] m Char => ParsecT [Char] JournalContext m ()
 multilinecommentp = do
-  string "comment" >> newline
+  string "comment" >> many spacenonewline >> newline
   go
   where
     go = try (string "end comment" >> newline >> return ())
@@ -906,7 +950,10 @@ followingcommentp =
      return $ unlines $ samelinecomment:newlinecomments
 
 comment :: Stream [Char] m Char => ParsecT [Char] JournalContext m String
-comment = commentStartingWith "#;"
+comment = commentStartingWith commentchars
+
+commentchars :: [Char]
+commentchars = "#;*"
 
 semicoloncomment :: Stream [Char] m Char => ParsecT [Char] JournalContext m String
 semicoloncomment = commentStartingWith ";"
@@ -979,9 +1026,13 @@ dateValueFromTags  ts = maybe Nothing (Just . snd) $ find ((=="date") . fst) ts
 date2ValueFromTags ts = maybe Nothing (Just . snd) $ find ((=="date2") . fst) ts
 
 
+tests_Hledger_Read_JournalReader = TestList $ concat [
+    -- test_numberp
+ ]
+
 {- old hunit tests
 
-test_Hledger_Read_JournalReader = TestList $ concat [
+tests_Hledger_Read_JournalReader = TestList $ concat [
     test_numberp,
     test_amountp,
     test_spaceandamountormissing,
